#ifndef PARTICLE_H
#define PARTICLE_H

#include "bfastVector.H"
#include <vector>
#include <cstdlib>

class Particle {
public:
  Particle() {};
  ~Particle() {};
  inline Particle(const BfastVector3 &pos, const BfastVector3 &vel,
		  const BfastVector3 &acc, const BfastVector3 &col,
		  const double m, const double size, const double r,
		  const double crest, const double fricNum, const double age);
  BfastVector3 pos, vel, acc, col;
  double m, size, r, crest, fricNum, age;
};

class Obstacle {
public:
  Obstacle() {};
  virtual ~Obstacle() {};
  virtual bool bounce(Particle &p) const = 0;
};

class PlaneObstacle : Obstacle {
public:
  BfastVector3 n;
  double off;
  PlaneObstacle() {};
  PlaneObstacle(const BfastVector3 &n, double off) : n(n), off(off) {};
  ~PlaneObstacle() {};
  bool bounce(Particle &p) const;
};

bool step(std::vector<Particle> &particles, std::vector<Obstacle*> &obstacles, double dt);

inline Particle::Particle(const BfastVector3 &pos, const BfastVector3 &vel,
			  const BfastVector3 &acc, const BfastVector3 &col,
			  const double m, const double size, const double r,
			  const double crest, const double fricNum, const double age) :
  pos(pos), vel(vel), acc(acc), col(col), m(m), size(size), r(r), crest(crest), fricNum(fricNum), age(age){};

#endif
