#include "particle.H"
#include <vector>

bool step(std::vector<Particle> &particles, std::vector<Obstacle*> &obstacles, double dt) {
  std::vector<Particle>::iterator p,q;
  std::vector<Obstacle*>::iterator o;
  for (p=particles.begin(); p!=particles.end(); p++) {
    // initialize acceleration
    p->acc = 0.0;
    //apply per particle forces
    // gravity
    p->acc += BfastVector3(0,0,-9.8)*p->m;
    
		// interparticle forces
    for (q=particles.begin(); q!=particles.end(); q++) {
			// none for now.
    }
	}
  
  for (p=particles.begin(); p!=particles.end(); p++) {
		// integrate
    p->age += dt;
    p->acc /= p->m;
    BfastVector3 newVel = p->vel + dt*p->acc;
    p->pos += 0.5*dt*(p->vel+newVel);
    p->vel = newVel;
    
    // check for collisions
    for (o=obstacles.begin(); o!=obstacles.end(); o++) {
      (*o)->bounce(*p);
    }
  }
  
  return true;
}

double sign(double x) {
  if (x < 0.0) return -1.0;
  else return 1.0;
}

bool PlaneObstacle::bounce(Particle &p) const {
  double d = dot(p.pos,n);
  if (d > off) return false;
  p.pos += (off-d)*n;
  double normMag = dot(p.vel,n);
  BfastVector3 normVel = normMag*n;
  BfastVector3 tanVel = p.vel - normVel;
  p.vel = p.crest * sign(normMag) * normVel + (1 - p.fricNum) * tanVel;
  return true;
}
